
from kraken_html.html_bootstrap import Html_bootstrap as HTML
from kraken_html.kraken_record_html import Kraken_record_html as KR_HTML

import json
import copy

import requests

#from models.html_form import Html_form




class Kraken_html:
    """ Class to generate html pages based on standard kraken records

    Attributes:
    - get_page: Gets a full website page with default record page layout
    - 

    Parts:
    - get_article: 
    - get_product:
    - get_record:
    - get_image
    - get_video


    """

    def __init__(self, organization = '', title = None, description = '', nav = '', footer_nav = '', logo = ''):


        self.organization = organization
        self.title = title
        self.description = description
        self.nav = nav
        self.footer_nav = footer_nav
        self.logo = logo

        self.html = HTML(self.title, self.description, self.nav, self.footer_nav, self.logo)

        """
        config_file = {
            'organization': 'Org name',
            'title': 'Title of website',
            'description': 'Some description of the website',
            'nav': [
                {'url': '/home', 'text': 'Home'},
                {'url': '/products', 'text': 'Products'},
                {'url': '/about', 'text': 'About us'},
                {'url': '/contact', 'text': 'Contact us'}
                ],
            'footer_nav': [
                {'url': '/privacy', 'text': 'Privacy policy'},
                {'url': '/terms', 'text': 'Terms'},
                {'url': '/about', 'text': 'About us'},
                {'url': '/contact', 'text': 'Contact us'}
                ],
            'logo': 'https://cdn.icon-icons.com/icons2/1678/PNG/512/wondicon-ui-free-home_111275.png',
            'read_db': 'https://storage.googleapis.com/krkn/v1'
            }
        """
        


    def config(self, config_record):

        self.organization = config_record.get('organization', '')
        self.title = config_record.get('title', '')
        self.description = config_record.get('description', '')
        self.nav = config_record.get('nav', '')
        self.footer_nav = config_record.get('footer_nav', '')
        self.logo = config_record.get('logo', ''),
        self.read_db = config_record.get('read_db', '')

        self.html = HTML(self.title, self.description, self.nav, self.footer_nav, self.logo)
        
    def get_db_record(self, record_type, record_id):
        # Retrieves a record from kraken db

        url = self.read_db + '/' + record_type + '/' + record_id + '.json'

        try:
            r = requests.get(url)

            record = json.loads(r.content)

        except:
            print(r.content)
            record = None

        return record


    def get_records_page(self, records, breadcrumbs = ''):
        # Shows page with multiple record

        title = 'Title'

        schema_content = ''


        records_content = self.get_cards(records)

        # Combine parts
        body_content = schema_content + records_content 


        # Insert in web page
        page_content = self.html.page_blank(title, body_content, breadcrumbs)


        return page_content


    def get_record_page(self, record, breadcrumbs = ''):
        # Returns full web page with relevant web part for the record
        '''
        Page structure:
        - page_content:
            - jsonld_content
            - body_content:
                - part1_content
                - part2_content

        '''


        kr = KR_HTML(record)

        # If invalida record, send error message
        if not kr.valid:
            title = 'Invalid record'
            body_content = str(record)
            return self.html.page_blank(title, body_content, breadcrumbs)



        # Retrieve jsonld
        jsonld_content = kr.jsonld


        # Retrieve web part usinf proper record style

        if kr.layout == 'article':

            part_content = self.get_article(record, breadcrumbs)

        elif kr.layout == 'image':

            part_content = self.get_page_image(record, breadcrumbs)


        elif kr.layout == 'product':

            part_content = self.get_page_product(record, breadcrumbs)

        else:
            part_content = self.get_record(record, breadcrumbs)




        # Combine parts
        body_content = jsonld_content + part_content 


        # Insert in web page
        page_content = self.html.page_blank(kr.name, body_content, breadcrumbs)


        return page_content



    def get_article(self, record, breadcrumbs = ''):
        # Return a full page with article template html of the record
        '''
        article:
            - title
            - author
            - image
            - body text
        '''

        kr = KR_HTML(record)

        # Retrieve web part
        part_content = self.html.article(kr.headline + 'f', kr.image, kr.articlebody)

        return part_content


    def get_image(self, record, breadcrumbs = ''):
        # Return a full page with article template html of the record
        
        # Extarct relevant data from record
        title = record.get('schema:headline', '')
        articlebody = record.get('schema:articlebody', '')
        image = record.get('schema:image', '')

        return self.html.page_image(title, image, articlebody, breadcrumbs)

    def get_product(self, record, breadcrumbs = ''):
        # Return a full page with article template html of the record
        
        # Extarct relevant data from record
        title = record.get('schema:headline', '')
        articlebody = record.get('schema:articlebody', '')
        image = record.get('schema:image', '')

        return self.html.page_product(title, image, articlebody, breadcrumbs)
    

    def get_record(self, record, breadcrumbs = ''):
        # Return a full page with article template html of the record
        
        kr = KR_HTML(record)


        # Retrieve picture (if one)
        print(kr.image)
        image_content = self.html.thumbnail(kr.image)

        # Retrieve web part
        record_content = self.html.record(kr.name, kr.html_record)


        # Combine parts
        body_content = image_content + record_content 


        return body_content




    def get_cards(self, records):
        # Return html cards

        if not isinstance(records, list):
            records = [records]

        cards = []
        for i in records:
            kr = KR_HTML(i)
            image_url = kr.image
            link_url = kr.url
            name = kr.name

            cards.append(self.html.card(image_url, link_url, name))

        cards_html = self.html.cards(cards)

        return cards_html



    def get_record_template(self, page, language = 'en'):
        # Get file Name. 
        filename = 'content_template/' + page + '_' + language + '.json'

        # Load file
        f = open(filename, "r")
        content = f.read()

        # COnvert \n to <br>
        content = content.replace('\\n', '<br>')


        # Change placeholders

        content = content.replace('{organization}',self.organization)



        # Convert to json
        record = json.loads(content)


        # Return
        return record





    def _get_keys(self, value):

        if not isinstance(value, list):
            value = [value]

        keys = []

        for i in value:
            for key in i:
                if key not in keys:
                    keys.append(key)

        return sorted(keys)




import copy
import json

class Kraken_record_html:
    # Converts normal schema record to html usable record

    def __init__(self, record):

        self.record = record


    @property
    def valid(self):
        # Test if valid record

        if not isinstance(self.record, dict):
            return False

        if not self.record_type:
            return False

        if not self.record_id:
            return False

        return True



    @property 
    def record_type(self):
        # Return record type

        #Error handling
        if not isinstance(self.record, dict):
            return None

        # Process
        return self.record.get('@type', '')


    @property
    def record_id(self):
        # Return record type

        #Error handling
        if not isinstance(self.record, dict):
            return None
        
        #Process
        return self.record.get('@id', '')


    @property
    def url(self): 
        # Return relative url for the record

        # Build url
        url = '/' + self.record_type + '/' + self.record_id

        return url


    @property
    def headline(self):
        return self._get_subrecord_attr('schema:headline', 'link')

    @property
    def author(self):
        return self._get_subrecord_attr('schema:headline', 'link')

    @property
    def articlebody(self):
        return self._get_subrecord_attr('schema:articlebody', 'link')

    @property
    def email(self):
        return self._get_subrecord_attr('schema:email', 'link')




    @property
    def name(self):
        # Gets the name of the record base don type and availability

        name = self.record.get('schema:name', None)
        email = self.record.get('schema:email', None)
        headline = self.record.get('schema:headline', None)


        if self.record_type == 'schema:article' and headline:
            record_name = headline
        elif name:
            record_name = name
        elif self.record_type == 'schema:person' and email:
            record_name = email
        else:
            record_name = self.record_id

        # If list, take first item
        if isinstance(record_name, list):
            record_name = record_name[0]

        return record_name

    @property
    def link(self):
        # Generate html link

        link = '<a href="{url}">{name}</a>'.format(url=self.url, name = self.name)
        
        return link

    @property
    def link_thumbnail(self):
        # Generate html link

        link = '<a href="{url}">{name}</a>'.format(url=self.url, name = self.name)
        
        return link


    @property
    def image(self):
        # Retrieves the url for the media picture
        # Priority:
        # thumbnail, content

        # Error handling
        if not isinstance(self.record, dict):
            return None


        record_thumbnailurl = self.record.get('schema:thumbnailurl', None)
        record_contenturl = self.record.get('schema:contenturl', None)
        record_image = self.record.get('schema:image', None)


        # Assign picture
        if record_thumbnailurl:
            picture_url = record_thumbnailurl
        elif record_contenturl:
            picture_url = record_contenturl
        else:
            picture_url = record_image

        if isinstance(picture_url, list):
            picture_url = picture_url[0]


        return picture_url


    @property
    def html_record(self, thumbnail = True):
        # Return record with sub-records as html links
        # Thumbnail flag activate / deactivate adding thumbnail

        # Create a copy so not to change original
        record = copy.deepcopy(self.record)
        new_record = {}

        # Loop through keys
        for i in record:

            # Convert to list
            if not isinstance(record[i], list):
                record[i] = [record[i]]

            # Iterate through list
            values = []
            for t in record[i]:

                # Test if valid record
                k = Kraken_record_html(t)
                if k.valid: 
                    value = k.link
                else:
                    value = t

                values.append(value)
            

            # Convert back to single item if needed. 
            if len(values) == 1:
                values = values[0]


            new_record[i] = values

        return new_record


    @property
    def layout(self):
        # Returns the page layout to use based on the record type

        if self.record_type in ['schema:article']:

            return 'article'

        elif self.record_type in ['schema:product']:

            return 'product'

        elif self.record_type in ['schema:imageobject']:

            return 'image'

        else:

            return 'record'


    @property 
    def jsonld(self):
        # Takes a record and returns a <scripys> jsonld type html code 
        
        schema_text = json.dumps(self.record, default=str, indent = 4)

        content = '\n<script type="application/ld+json">\n' + schema_text + '\n</script>\n'


        return content

    def _get_subrecord_attr(self, key, attr = None):
        # Retrieves a given fields from a sub-record. Returns list of values if several sub records

        sub_records = self.record.get(key, '')

        if not isinstance(sub_records, list):
            sub_records = [sub_records]

        values = []
        for i in sub_records:
            sub_kr = Kraken_record_html(i)
            if attr and sub_kr.valid:
                value = getattr(sub_kr, attr)
            else: 
                value = i

            values.append(value)

        # De list if needed
        if len(values) == 1:
            values = values[0]

        return values


from os import listdir

HTML_TEMPLATE_PATH = 'html_template'

html_templates = {}


class Html_bootstrap:
    """ Class to obtain html snippets in bootstrap format

	LONG DESCRIPTION OF CLASS

	ATTRIBUTES:
		ATTRIBUTE1(type): Description
		ATTRIBUTE2(type): Description
	"""

    def __init__(self, title = '', description = '', nav = '', footer = '', logo = ''):
        # Handles calling the ootstrap componenets required
        a = 1

        self.title = title
        self.description = description
        self.nav = nav
        self.footer_items = footer
        self.logo = logo

        if not html_templates:
            self.load_html_templates()

    def load_html_templates(self):

        files =listdir(HTML_TEMPLATE_PATH)

        # load html templates into memory
        for i in files:
            item = i.replace('.txt', '')
            filename = HTML_TEMPLATE_PATH + '/' + i
            f = open(filename, "r")
            html_templates[item] = f.read()
        return




    def heading(self, text, level = 'h1'):
        # Returns a heading with the inserted text
        # Level is leveh of heading (h1, h2)
        # Text is actual text to insert. 

        html_content = html_templates['heading']
        html_content = html_content.replace('{text}', text)
        html_content = html_content.replace('{level}', level)

        return html_content


    def text(self, text):
        # Return a paragraph

        return '<p>' + text + '</p>'


    def link(self, text, url):
        # Return a link
        html_content = html_templates['link']
        html_content = html_content.replace('{text}', text)
        html_content = html_content.replace('{url}', url)
        return html_content

    def image(self, contenturl): 

        if not contenturl:
            return ''

        content = '<img src="' + contenturl + '" class="img-fluid" alt="...">'
        return content


    def thumbnail(self, contenturl = '', linkurl = ''): 

        if not contenturl:
            return ''

        content = '<img src="' + contenturl + '" class="w-25" alt="...">'

        content = self.link(content, linkurl)

        return content


    def list(self, title, items, style_class_list = 'list-unstyled', style_class_item = ''):
        # Create list

        content = ''
        # Init list
        content += '<ul class="{style_class_list}">'.format(style_class_list = style_class_list)

        # Add items
        for i in items:
            content += '<li class="{style_class_item}">{item}</li>'.format(item = i, style_class_item=style_class_item)

        # Finalize list
        content += '</ul>'

        return content

    def breadcrumb(self, breadcrumb_items):
        # Returns breadcrumb html from list of url, text dicts.

        if not breadcrumb_items:
            return ''


        content = ''

        # Get past pages
        if len(breadcrumb_items) > 1:
            past_breadcrumb_items = breadcrumb_items[0:-1]

            for i in past_breadcrumb_items:
                url = i.get('url', '')
                text = i.get('text', '')

                content += '<li class="breadcrumb-item"><a href="{url}">{text}</a></li>'.format(url=url, text=text)

        # Get current pages
        record = breadcrumb_items[-1]
        url = record.get('url', '')
        text = record.get('text', '')
        content += '<li class="breadcrumb-item"><a href="{url}">{text}</a></li>'.format(url=url, text=text)


        html_content = html_templates['breadcrumb']
        html_content = html_content.replace('{breadcrumb_list}', content)
        return html_content


        return content

    def record(self, title = '', record = ''):
        # Return a web part for the record

        if not title: 
            title = ''


        # Build record
        record_content = ''
        for i in record:
            # Add key
            record_content +=  '<dt class="col-sm-3">'
            record_content +=  str(i) 
            record_content += ':</dt>'

            # Add values

            record_content += '<dd class="col-sm-9">'

            values = record.get(i, [])
            if not isinstance(values, list):
                values = [values]

            record_content += self.list('', values)

            record_content += '</dd>'


        # Build html

        base_content = html_templates['record']
        
        html_content = base_content.replace('{title}', title)

        html_content = base_content.replace('{record_content}', record_content)
        
        # Add title
        if title:
            title_content = self.heading(title)
            html_content = title_content + html_content
        
        
        return html_content




    def table(self, title, items, keys = []):
        # Returns a html table. Items are list of dicts. If keys is provided, only shows these columns. 

        # Get all keys from items if not provided
        if not keys:
            keys = []
            for i in items:
                for k in i:
                    keys.append(k)


        # Make header
        table_header = ''
        for i in keys:
            table_header += '<th scope="col">' + i + '</th>'
                    
        # MAke body
        table_body = ''
        for i in items:
            table_body += '<tr>'
            for k in keys:
                value = i.get(k, '')
                table_body += '<td>' + str(value) + '</td>'
      
            table_body += '</tr>'
      
        # Build table
        html_content = html_templates['table']
        html_content = html_content.replace('{table_header}', table_header)
        html_content = html_content.replace('{table_body}', table_body)
        return html_content



    def navbar(self, title, menu_items):
        # Returns a navbar

        if not title:
            title = ''

        # Build links
        list_items = []
        for i in menu_items:
            text = i.get('text', '')
            url = i.get('url', '')

            html_content = html_templates['navbar_item']
            html_content = html_content.replace('{url}', url)
            html_content = html_content.replace('{text}', text)
            list_items.append(html_content)


        # Convert links to list and assign style 'navbar-nav'
        menu_list_html = self.list('', list_items, 'navbar-nav', 'nav-item')

        html_content = html_templates['navbar']
        html_content = html_content.replace('{title}', title)
        html_content = html_content.replace('{menu_items}', menu_list_html)

        return html_content



    def footer(self, title, description, footer_items):
        # Returns a footer

        if not title:
            title = ''

        # Build links
        list_items = []
        for i in footer_items:
            text = i.get('text', '')
            link = i.get('link', '')

            list_items.append(self.link(text, link))

        # Convert links to list and assign style 'navbar-nav'
        footer_items_list_html = self.list('', list_items)


        html_content = html_templates['footer']
        html_content = html_content.replace('{title}', title)
        html_content = html_content.replace('{description}', description)

        html_content = html_content.replace('{footer_items}', footer_items_list_html)

        return html_content


    def article(self, title, image, content):
        # Returns an article web part
        
        page_content = ''
        page_content += self.heading(title)
        page_content += self.image(image)
        page_content += self.text(content)

        return page_content


    def page_blank(self, title, page_content, breadcrumb_items):

        # Returns a heading with the inserted text
        # Level is leveh of heading (h1, h2)
        # Text is actual text to insert. 

        if not title:
            title = ''


        nav_html = self.navbar(self.title, self.nav)
        
        page_footer_html = self.footer(self.title, self.description, self.footer_items)

        breadcrumb_html = self.breadcrumb(breadcrumb_items)

        html_content = html_templates['page']
        html_content = html_content.replace('{title}', title)
        html_content = html_content.replace('{nav}', nav_html)
        html_content = html_content.replace('{breadcrumb}', breadcrumb_html)

        html_content = html_content.replace('{content}', page_content)
        html_content = html_content.replace('{footer}', page_footer_html)

        return html_content


    def page_article(self, title, image, content, breadcrumb):
        # Returns a full web page including article web part

        body_content = self.article(title, image, content)
        
        return self.page_blank(title, body_content, breadcrumb)



    def card(self, image_url = '', link_url = '', title = '', text = ''):
        # Returns a card with the inserted text
        # Level is leveh of heading (h1, h2)
        # Text is actual text to insert. 

        html_content = html_templates['card']
        html_content = html_content.replace('{link_url}', link_url)
        html_content = html_content.replace('{card_title}', title)
        html_content = html_content.replace('{image_url}', image_url)
        html_content = html_content.replace('{card_text}', text)

        return html_content


    def cards(self, items):


        content = '<div class="row row-cols-1 row-cols-sm-4 g-4">'
        for item in items:

            content += '<div class="col">' + item + '</div>'
        content += '</div>'

        return content

    def section(self, title, content, expanded = False):
        # Expandable section
        
        # Remove non aascii characters for system title
        system_title = title.replace(' ', '')
        system_title = title.encode("ascii", "replace")
        system_title = 'section_' + system_title.decode("ascii", "replace").lower()
        system_title = title.replace(' ', '')


        # Convert true false in string
        js_expanded = str(expanded).lower()



        html_content = html_templates['section']
        
        html_content = html_content.replace('{title}', title)
        html_content = html_content.replace('{system_title}', system_title)
        html_content = html_content.replace('{content}', content)
        html_content = html_content.replace('{expanded}', js_expanded)

        return html_content





class Records:

    def __init__(self):

        a=1

    @property
    def record(self):

        record = {
            '@id': 'test_record_id',
            '@type': 'Test_record_type',
            'schema:name': 'test_name',
            'schema:url': 'https://www.test/com',
            'schema:email': 'test@test.com',
            'schema:givenname': 'test_given',
            'schema:familyname': 'test_family',
            'schema:contenturl': 'https://static.independent.co.uk/s3fs-public/thumbnails/image/2014/11/03/14/Kitten.jpg?width=1368'

        }

        return record


    @property
    def records(self):

        i = 0
        records = []

        while i < 10:
            record = self.record

            for p in record:
                if p != 'schema:contenturl':
                    record[p] = record[p] + str(i)

            records.append(record)
        
            i += 1

        return records


from dateutil.parser import parse



class Html_field:

    def __init__(self):

        a = 1


    def get(self, record_type, record_id, key, value):

        content = ''
        path = None

        # Assign path for record if both exist
        if record_type and record_id:
            path = '/' + record_type + '/' + record_id



        # Check if object, if so puts a link to it

        if isinstance(value, dict):
            sub_type = value.get('@type', '')
            sub_id = value.get('@id', '')
            sub_path = '/' + sub_type + '/' + sub_id

            content = self.format_link(sub_id, sub_path)


        # Check if date, if so format it. 
        elif 'date' in key.lower():
            content = self.format_date(value)

        # Check if id or name of record 
        elif path and key in ['@id', 'schema:name']:

            content = self.format_link(value, path)

        # Check if url
        elif path and key in ['schema:url', 'schema:contenturl']:

            content = self.format_link(value, value)
        
        # Check if phone
        elif path and key in ['schema:phone', 'schema:telephone', 'schema:fax']:

            content = self.format_phone(value, value)

        # Check if email
        elif path and key in ['schema:email']:

            content = self.format_email(value, value)
        
        # Check if sms
        elif path and key in ['schema:sms']:

            content = self.format_sms(value, value)

        # Check if number
        elif self.format_number(value):
            content = self.format_number(value)



        else:
            content = value

        return content






    def format_number(self, value):
        # Convert value to right aligned 

        content = None
        i_value = None

        try:
            i_value = int(value)

        except:
            try: 
                i_value = float(value)
            except:
                a = 1

        if i_value:
            content = '<p style="text-align:right;">{value}</p>'.format(value=str(i_value))

        return content


    def format_sms(self, value, link):
        # Convert value to html link

        content = '<a href="sms:{link}">{value}</a>'.format(value=str(value), link=str(link))

        return content


    def format_email(self, value, link):
        # Convert value to html link

        content = '<a href="mailto:{link}">{value}</a>'.format(value=str(value), link=str(link))

        return content



    def format_phone(self, value, link):
        # Convert value to html link

        content = '<a href="tel:{link}">{value}</a>'.format(value=str(value), link=str(link))

        return content



    def format_link(self, value, link):
        # Convert value to html link

        content = '<a href="{link}">{value}</a>'.format(value=str(value), link=str(link))

        return content


    def format_date(self, value):
        # Check if value if of type date and formats it if so.

        try:
            a = parse(str(value))
            value = a.strftime("%Y-%m-%d, %H:%M")

        except:
            a = 1

        return value